package cd.casic.ci.process.process.service.report.impl;

import cd.casic.ci.process.dal.req.report.ReportDeleteReq;
import cd.casic.ci.process.dal.resp.report.ReportResp;
import cd.casic.ci.process.engine.context.ConstantContextHolder;
import cd.casic.ci.process.process.dataObject.base.BaseIdPageReq;
import cd.casic.ci.process.process.dataObject.base.BaseIdReq;
import cd.casic.ci.process.process.service.report.ReportService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.pojo.PageResult;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import static cd.casic.ci.process.engine.worker.base.HttpWorker.getRestTemplateWithoutSANCheck;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName TargetVersionServiceImpl
 * @Date: 2025/5/17 15:20
 * @Description:
 */
@Service
@Slf4j
public class ReportServiceImpl implements ReportService {

    public static final String PATH = "/openapi/v1/asset/report/downLoadReport/batch";

    @Override
    public PageResult<ReportResp> findReportList(BaseIdPageReq req) throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        RestTemplate restTemplate = getRestTemplateWithoutSANCheck();
        String reportFindUrl = ConstantContextHolder.getScaIp() + "/openapi/v1/asset/report/list";
        String body = buildFindRequestBody(req);
        HttpHeaders headers = createHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<String> requestEntity = new HttpEntity<>(body, headers);

        log.info("报告分页查询接口:" + reportFindUrl);
        JSONObject response = restTemplate.postForObject(reportFindUrl, requestEntity, JSONObject.class);
        String message = response.getString("message");
        if (message.equals("success")) {
            List<ReportResp> respList = new ArrayList<>();
            log.info("===============报告分页查询接口成功=================");
            JSONObject data = response.getJSONObject("data");
            setReportFindResp(data,response.getString("xmTraceId"),respList);

            PageResult<ReportResp> pageResult = new PageResult<>(respList,data.getLong("total"),data.getLong("pageNum"),data.getLong("pageSize"));
            return pageResult;

        } else {
            log.error("===============报告分页查询接口失败=================");
            return new PageResult<>();
        }
    }



    @Override
    public ReportResp deleteReport(ReportDeleteReq req) throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        RestTemplate restTemplate = getRestTemplateWithoutSANCheck();
        String reportDeleteUrl = ConstantContextHolder.getScaIp() + "/openapi/v1/asset/report/delete";
        String body = buildDeleteRequestBody(req);
        HttpHeaders headers = createHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<String> requestEntity = new HttpEntity<>(body, headers);

        log.info("报告删除接口:" + reportDeleteUrl);
        JSONObject response = restTemplate.postForObject(reportDeleteUrl, requestEntity, JSONObject.class);
        String message = response.getString("message");
        if (message.equals("success")) {
            ReportResp reportResp = new ReportResp();
            log.info("===============报告删除接口成功=================");
            JSONObject data = response.getJSONObject("data");
            setReportDeleteResp(data,response.getString("xmTraceId"),reportResp);

            return reportResp;

        } else {
            log.error("===============报告删除接口失败=================");
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"删除报告失败");
        }
    }



    @Override
    public ResponseEntity<byte[]> downLoadReport(BaseIdReq req) throws NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        RestTemplate restTemplate = getRestTemplateWithoutSANCheck();
        String reportDeleteUrl = ConstantContextHolder.getScaIp() + PATH;
        String body = buildDownloadRequestBody(req);
        HttpHeaders headers = createHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<String> requestEntity = new HttpEntity<>(body, headers);

        log.info("报告下载接口:" + reportDeleteUrl);
        ResponseEntity<byte[]> responseEntity = restTemplate.postForEntity(reportDeleteUrl, requestEntity, byte[].class);

        if (responseEntity.getStatusCode() == HttpStatus.OK && responseEntity.hasBody()) {
            log.info("===============报告下载接口成功=================");

            // 构建返回给前端的 ResponseEntity<byte[]>
            return ResponseEntity.ok()
                    .contentType(MediaType.APPLICATION_OCTET_STREAM)
                    .header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=report.bin")
                    .body(responseEntity.getBody());
        }else {
            log.error("===============报告下载接口失败=================");
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(), "删除报告失败");
        }
    }




    private HttpHeaders createHeaders() {
        HttpHeaders headers = new HttpHeaders();
        headers.add("OpenApiUserToken", ConstantContextHolder.getScaToken());
        return headers;
    }

    private String buildFindRequestBody(BaseIdPageReq req) {
        JSONObject json = new JSONObject();
        if (req.getProjectId() != null) {
            json.put("projectId", req.getProjectId());
        }
        json.put("pageNum", req.getPageNo());
        json.put("pageSize", req.getPageSize());
        return json.toJSONString();
    }

    private String buildDeleteRequestBody(ReportDeleteReq req) {
        JSONObject json = new JSONObject();

        json.put("endDate", req.getEndDate());
        json.put("name", req.getName());
        json.put("projectIdList", req.getProjectIdList());
        json.put("reportId", req.getReportId());
        json.put("sourceCode", req.getSourceCode());
        json.put("startDate", req.getStartDate());
        json.put("status", req.getStatus());
        json.put("type", req.getType());
        json.put("use", req.getUse());

        return json.toJSONString();
    }

    private String buildDownloadRequestBody(BaseIdReq req) {
        JSONObject json = new JSONObject();
        json.put("reportIds", req.getReportIds());
        return json.toJSONString();
    }


    private void setReportFindResp(JSONObject data, String xmTraceId, List<ReportResp> respList) {
        JSONArray records = data.getJSONArray("records");
        for (int i = 0; i < records.size(); i++) {
            JSONObject report = records.getJSONObject(i);
            ReportResp reportResp = new ReportResp();

            reportResp.setId(report.getInteger("id"));
            reportResp.setCreateDate(report.getString("createDate"));
            reportResp.setCreator(report.getString("creator"));
            reportResp.setDimension(report.getInteger("dimension"));
            reportResp.setMd5(report.getString("md5"));
            reportResp.setName(report.getString("name"));
            reportResp.setProjectName(report.getString("projectName"));
            reportResp.setSerialNumber(report.getString("serialNumber"));
            reportResp.setSource(report.getInteger("source"));
            reportResp.setSourceName(report.getString("sourceName"));
            reportResp.setStatus(report.getInteger("status"));

            reportResp.setXmTraceId(xmTraceId);

            List<Integer> typeList = new ArrayList<>();
            JSONArray typeArray = report.getJSONArray("type");
            for (int j = 0; j < typeArray.size(); j++) {
                typeList.add(typeArray.getInteger(j));
            }
            reportResp.setType(typeList);

            respList.add(reportResp);
        }
    }


    private void setReportDeleteResp(JSONObject data, String xmTraceId, ReportResp reportResp) {
        reportResp.setFail(data.getInteger("fail"));
        reportResp.setSuccess(data.getInteger("success"));
        reportResp.setXmTraceId(xmTraceId);
    }


}
